#ifndef _NEMO_VECTOR_H_
#define _NEMO_VECTOR_H_


#include "IntrusiveCowSupport.h"
#include "IntrusiveCounted.h"
#include "Collection.h"
#include "nemo-cpp11.h"
#include "Random.h"

#include <stdexcept>



/*! \mainpage NemoMath Overview
 *
 * \section Introduction
 *
 * A header-only template library for mathematical computations in the context of robotics and learning.
 *  - Matrix and Vector data types, optimized for shared-ownership in object-oriented applications by means of "Copy on write" and intrusive-pointering
 *  - Neat functional programming API to generate and use Functor objects
 * 
 *
 * \section la Linear Algebra data types
 * 
 * The MathVector and Matrix classes provide standard linear algebra functionality.
 * They are optimized for shared ownership in complex object-oriented application.
 * Therefore they comprise a combination of efficient intrusive-pointer mechanisms and copy-on write.
 * @see Vector.h Matrix.h
 * @see MatrixMath.h
 * 
 * \section func Functor library
 * 
 * NemoMath provides a very convenient way to express and utilize mathematical function objects (functors).
 * You can define your own function objects or just use built in operators to express complex mathematical
 * relations. The Collection API allows these functors to smoothly interoperate with the Matrix and Vector data types.
 * @see Mapping.h Collection.h
 * @see ScalarMappings.h VectorMappings.h Norms.h Filters.h
 * 
 * \section cont Other mathematical data-structures
 * 
 * NemoMath provides container data types for more complex structures. Currently it provides an 
 * abstraction for time series.
 * @see TimeSeries.h
 *  
 */




namespace nemo {


class VectorDimension;

static VectorDimension dim(unsigned int d);

/**
 * Represents the dimension of a MathVector. It is actually nothing else than an unsigned int,
 * but allows typesafe distinction between values and dimensions in MathVectors.
 */
class VectorDimension {
public:
	inline const unsigned int& get() const {return d;}
	friend VectorDimension dim(unsigned int d);
	operator unsigned int () const {
		return d;
	}
private:
	VectorDimension(unsigned int dd) : d(dd) {}
	//VectorDimension(VectorDimension &o) : d(o.d) {}
	//VectorDimension& operator=(const VectorDimension& o){d = o.d; return *this;}
	unsigned int d;
};

/**
 * Neat factory method for VectorDimension objects to be put into MathVector objects.
 */
static inline VectorDimension dim(unsigned int d){
	return VectorDimension(d);
}

template <typename T>
class VectorTranspose;

/**
 * This class represents a mathematical vector of some data type T.
 * This can for instance be T=double, but all types are applicable 
 * that define:
 * - default-constructor, copy-constructor and operator=() 
 * - numerical operators +, -, *, /
 * - equality operators ==, !=
 * 
 * Each vector has a certain dimension, which is returned by the dimension() method.
 * The vector contains that many values of type T which can be accessed by an index
 * between zero and dimension-1.
 * The class provides basic access and manipulation of these values with the [] operator:
 * MathVector<int> x(dim(3), 2);
 * int two = x[0];
 * for(unsigned int i=0; i<x.dimension(); i++){
 * 	x[i] = 42;
 * }
 */
template <class T>
class MathVector : public Collection<T> {

	
#ifdef NEMO_ABORT_ON_ERROR
	#define _NEMO_VECTOR_CHECK_INDEX(index) \
		if(data->dim <= index){ \
			abort(); \
		}
#else
	#define _NEMO_VECTOR_CHECK_INDEX(index) \
		if(data->dim <= index){ \
			throw std::invalid_argument("MathVector: index out of range"); \
		}
#endif


private:
	friend class VectorTranspose<T>; 

public:
	//class CWise;
	friend class CWise;  // CWise needs to access CWiseOperation classes

private:

	class VectorData : public IntrusiveCounted<VectorData > {
	public:
		VectorData(const unsigned int dimension)
		 : IntrusiveCounted<VectorData >(), dim(dimension), values(new T[dim])
		{}

		VectorData(const VectorData &vd)
		 : IntrusiveCounted<VectorData >(), dim(vd.dim), values(new T[dim])
		{
			for(unsigned int i=0; i<dim; i++){
				values[i] = vd.values[i];
			}
		}

		VectorData& operator=(const VectorData &r){
			if(this != &r){
				if(values) delete[](values);
				dim = r.dim;
				values = new T[dim];
				for(unsigned int i=0; i<dim; i++){
					values[i] = r.values[i];
				}
			}
			return *this;
		}

		~VectorData(){
			delete[](values);
		}
	public:
		unsigned int dim;
		T *values;
	};

public:
// =============================================================================
//    Construction
// =============================================================================

	/**
	 * Creates a vector with dimension 0.
	 */
	explicit MathVector()
	 : data(new VectorData(0))
	{}

	/**
	 * Creates a vector with dimension dim.
	 * All values inside are initialized with the default constructor,
	 * which, for instance, gives zeros for primitive numbers.
	 * 
	 * @param dim Dimension of the vector
	 */
	explicit MathVector(const VectorDimension &dim)
	 : data(new VectorData(dim.get()))
	{
		for(unsigned int i=0; i<dim.get(); i++){
			data->values[i] = T();
		}
	}

	/**
	 * Creates a vector with dimension dim, in which all values
	 * are initialized with value.
	 * 
	 * @param dim Dimension of the vector
	 * @param val Initial value of all entries 
	 */
	explicit MathVector(const VectorDimension &dim, const T &val)
	 : data(new VectorData(dim.get()))
	{
		for(unsigned int i=0; i<dim.get(); i++){
			data->values[i] = val;
		}
	}

	// FIXME maybe the template should be removed by typesafe Generator<T>&, avoid possible confusion with Vec(dim,val)
	/**
	 * Creates a vector with dimension dim, in which all values
	 * are initialized individually by the return value of gen().
	 * gen might, for instance be a random number generator from Random.h:
	 * MathVector<double> vec(dim(3), uniform<double>(0.0,1.0)))
	 * This creates a three-dimensional random vector with values between 0 and 1.
	 * 
	 * @param dim Dimension of the vector
	 * @param gen Value generator
	 */
	template <typename generatorType>
	explicit MathVector(const VectorDimension &dim, generatorType gen)
	 : data(new VectorData(dim.get()))
	{
		for(unsigned int i=0; i<dim.get(); i++){
			data->values[i] = gen();
		}
	}

	/**
	 * Create a one-dimensional vector with value [x].
	 */
	explicit MathVector(const T &x)
	 : data(new VectorData(1))
	{
		data->values[0] = x;
	}
	/**
	 * Create a two-dimensional vector with values [x,y].
	 */
	explicit MathVector(const T &x, const T &y)
	 : data(new VectorData(2))
	{
		data->values[0] = x;
		data->values[1] = y;
	}
	/**
	 * Create a three-dimensional vector with values [x,y,z].
	 */
	explicit MathVector(const T &x, const T &y, const T &z)
	 : data(new VectorData(3))
	{
		data->values[0] = x;
		data->values[1] = y;
		data->values[2] = z;
	}

#ifdef _NEMO_HAS_INIT_LISTS_
	MathVector(const std::initializer_list<T> &init)
	 : data(new VectorData(init.size()))
	{
		unsigned int i=0;
		for(const T *p = init.begin (); p != init.end (); ++p){
			data->values[i++] = *p;
		}
	}
#endif

	MathVector<T>& operator=(const MathVector<T> &v){
		this->data = v.data;
		return *this;
	}

	/**
	 * Creates a vector with dimension dim, and take over the values inside @vals@
	 * 
	 * @param dim Dimension of the vector
	 * @param vals Array with initial values
	 */
	template <typename D>	
    static MathVector<T> fromArray(const VectorDimension &dim, const D* vals) {
		MathVector<T> vec(dim);
		for(unsigned int i=0; i<dim.get(); i++){
			vec.data->values[i] = vals[i];
		}
		return vec;
	}

// =============================================================================
//    Access
// =============================================================================

	/**
	 * Returns the dimension of the vector, i.e. the number of contained elements.
	 */
	//unsigned int dimension() const {
	//	return data->dim;
	//}
	inline VectorDimension dimension() const {
		return dim(data->dim);
	}

	/**
	 * Returns a readable reference to the element at position index
	 * for index from [0:dimension()-1]
	 */
	const T& operator[](const unsigned int index) const{
		_NEMO_VECTOR_CHECK_INDEX(index);
		return data->values[index];
	}

	class ReadWriteRef {
	friend class MathVector;
	friend class MathVector::Iterator;
	private:
		ReadWriteRef(IntrusiveCowSupport<VectorData > &d, const unsigned int i) : data(d), index(i) {}
	public:	
		ReadWriteRef& operator=(const T &value){ 
			// write access
			data.detach();
			data->values[index] = value;
			return *this;
		}
		ReadWriteRef& operator=(ReadWriteRef &value){ 
			// write access
			data.detach();
			data->values[index] = (T)value;
			return *this;
		}

		operator const T() const{return data->values[index];}

		#define _nemo_mathvector_ref_define_compound_(OP) \
		ReadWriteRef& operator OP (const T& value){\
			data.detach();\
			data->values[index] OP value;\
			return *this;\
		}
		_nemo_mathvector_ref_define_compound_(+=)
		_nemo_mathvector_ref_define_compound_(-=)
		_nemo_mathvector_ref_define_compound_(*=)
		_nemo_mathvector_ref_define_compound_(/=)
		_nemo_mathvector_ref_define_compound_(%=)
		_nemo_mathvector_ref_define_compound_(&=)
		_nemo_mathvector_ref_define_compound_(|=)
		_nemo_mathvector_ref_define_compound_(^=)
		_nemo_mathvector_ref_define_compound_(<<=)
		_nemo_mathvector_ref_define_compound_(>>=)

		#define _nemo_mathvector_ref_define_compare_(OP) \
		bool operator OP (const typename MathVector<T>::ReadWriteRef& rhs) const{\
			return data->values[index] OP rhs.data->values[rhs.index];\
		}
		_nemo_mathvector_ref_define_compare_(==)
		_nemo_mathvector_ref_define_compare_(!=)
		_nemo_mathvector_ref_define_compare_(<=)
		_nemo_mathvector_ref_define_compare_(<)
		_nemo_mathvector_ref_define_compare_(>=)
		_nemo_mathvector_ref_define_compare_(>)

		/* gives wearnings not to return ReadWriteRef& 
		 * pragma only works globally *argh* 
		 *#pragma GCC diagnostic ignored "-Weffc++"
		 * standard signature would be
			FOO& FOO::operator ++();
			​FOO FOO::operator ++(int);
		 * but prefix operator must not return ref to itself, 
		 * that would behave like the postfix op. Rather, the type
		 * shuold an object of type T, but not writable (don't return T&!!!)
		 */
		/*
		// prefix
		T operator++(){
			data.detach();
			return ++(data->values[index]);
		}
		// postfix
		T operator++(int){
			data.detach();
			return (data->values[index])++;
		}
		
		// prefix
		T operator--(){
			data.detach();
			return --(data->values[index]);
		}
		// postfix
		T operator--(int){
			data.detach();
			return (data->values[index])--;
		}
		*/

		// this is a 'specialization' of std::swap that can be found by argument-dependent lookup.
		// this one is necessary for STL write operations to work correctly, since the standard swap
		// implementation does not work correctly on ReadWriteRef, since it already has reference semantics
		// note that arguments are passed by value (copy-construction does no internal write),
		// in order to permit passing of temp. values from operator[]
		friend inline void swap( typename nemo::MathVector<T>::ReadWriteRef a, typename nemo::MathVector<T>::ReadWriteRef b ){
			a.data.detach();
			b.data.detach();
			using std::swap;
			swap( a.data->values[a.index], b.data->values[b.index] );
		}
		friend inline T max( const typename nemo::MathVector<T>::ReadWriteRef& a, const typename nemo::MathVector<T>::ReadWriteRef& b ){
			using std::max;
			return max( a.data->values[a.index], b.data->values[b.index] );
		}
		friend inline T max( const typename nemo::MathVector<T>::ReadWriteRef& a, T b ){
			using std::max;
			return max( a.data->values[a.index], b );
		}
		friend inline T max( T a, const typename nemo::MathVector<T>::ReadWriteRef& b ){
			using std::max;
			return max( a, b.data->values[b.index] );
		}
		friend inline T min( const typename nemo::MathVector<T>::ReadWriteRef& a, const typename nemo::MathVector<T>::ReadWriteRef& b ){
			using std::min;
			return min( a.data->values[a.index], b.data->values[b.index] );
		}
		friend inline T min( const typename nemo::MathVector<T>::ReadWriteRef& a, T b ){
			using std::min;
			return min( a.data->values[a.index], b );
		}
		friend inline T min( T a, const typename nemo::MathVector<T>::ReadWriteRef& b ){
			using std::min;
			return min( a, b.data->values[b.index] );
		}
		
	private:
		IntrusiveCowSupport<VectorData > &data;
		unsigned int index;
	};


	/**
	 * Returns a reference to the element at position index
	 * for index from [0:dimension()-1].
	 * Can be used for both read and write access
	 */
	inline ReadWriteRef operator[](const unsigned int index){
		_NEMO_VECTOR_CHECK_INDEX(index);
		return ReadWriteRef(data, index); // FIXME assignment/copy???
	}


	/**
	 * Returns a vector of dimension dim that contains the elements
	 * [fromIndex ... fromIndex+dim-1] of this vector.
	 * Both (fromIndex) and (fromIndex+dim-1) must be in range of
	 * [0:dimension()-1].
	 */
	const MathVector<T> subVector(const unsigned int fromIndex, const VectorDimension &dim) const {
		if(data->dim == 0){
			if(fromIndex!=0 || dim.get()!=0){
				throw std::invalid_argument(
					"zero-dimensional vector can only be devided with subVector(0,0)");
			}
			return *this;
		}

		_NEMO_VECTOR_CHECK_INDEX(fromIndex);
		if(0 < dim.get()){
			_NEMO_VECTOR_CHECK_INDEX(fromIndex+dim.get()-1);
		}

		MathVector<T> sub(dim);
		for(unsigned int i=0; i<dim.get(); i++){
			sub.data->values[i] = data->values[i+fromIndex];
		}
		return sub;
	}

	void setSubVector(const unsigned int fromIndex, const MathVector<T> &vec){
		_NEMO_VECTOR_CHECK_INDEX(fromIndex+vec.dimension()-1);

		data.detach();
		for(unsigned int i=0; i<vec.dimension(); i++){
			data->values[fromIndex+i] = vec.data->values[i];
		}
	}

// =============================================================================
//    Comparison
// =============================================================================
	/**
	 * Returns true if dimensions and all contained values are equal (i.e. exactly equal 
	 * in terms of T::operator==(T&) ).
	 */
	bool operator==(const MathVector<T> &r) const{
		if(data->dim != r.data->dim){
			return false;
		}
		for(unsigned int i=0; i<data->dim; i++){
			if(data->values[i] != r.data->values[i]){
				return false;
			}
		}
		// else
		return true;
	}

	/**
	 * ! operator==
	 */
	bool operator!=(const MathVector<T> &r) const{
		return !operator==(r);
	}

// =============================================================================
//    Collection API
// =============================================================================

	unsigned int numElements() const {
		return dimension();
	}
	T getElement(unsigned int elementIndex) const{
		return (*this)[elementIndex];
	}
	void setElement(unsigned int elementIndex, T value){
		(*this)[elementIndex] = value;
	}

	void transform(Mapping<T,T> m){
		for(unsigned int i=0; i<dimension(); i++){
			(*this)[i] = m((*this)[i]);
		}
	}

	template <typename OutType>
	MathVector<OutType> map(Mapping<T,OutType> m) const {
		MathVector<OutType> out(dim(this->dimension()));
		for(unsigned int i=0; i<dimension(); i++){
			out[i] = m((*this)[i]);
		}
		return out;
	}

// =============================================================================
//    STL compliant iterator interface
// =============================================================================

	class Iterator{
	private:
		Iterator(const MathVector<T> *vecP, int indexP) : vec(vecP), index(indexP), rwRef(const_cast<MathVector<T>*>(vecP)->data,(unsigned int)indexP) {}
	public:
		Iterator(const Iterator& it) : vec(it.vec), index(it.index), rwRef(const_cast<MathVector<T>*>(it.vec)->data,(unsigned int)it.index) {}
		Iterator& operator=(const Iterator &it){
			if(this==&it){return *this;}
			this->vec = it.vec;
			this->index = it.index;
			this->rwRef.data = it.rwRef.data;
			this->rwRef.index = it.rwRef.index;
			return *this;
		}
	
		typedef int difference_type;
		typedef T value_type;
		typedef ReadWriteRef* pointer;
		typedef ReadWriteRef reference;
		//typedef std::bidirectional_iterator_tag iterator_category;
		typedef std::random_access_iterator_tag iterator_category;

		ReadWriteRef& operator*(){
			if(index<0 || (int)vec->dimension()<=index){throw std::logic_error("Iterator out of range");}
			rwRef.index = (unsigned int)index;
			return rwRef;
			//return (*const_cast<MathVector<T>*>(vec))[(unsigned int)index];
		}
		T operator*() const{
			if(index<0 || (int)vec->dimension()<=index){throw std::logic_error("Iterator out of range");}
			return vec->getElement((unsigned int)index);
		}
		ReadWriteRef& operator[](difference_type offset){
			if(index+offset<0 || (int)vec->dimension()<=index+offset){throw std::logic_error("Iterator out of range");}
			rwRef.index = (unsigned int)index+offset;
			return rwRef;
			//return (*const_cast<MathVector<T>*>(vec))[(unsigned int)index+offset];
		}
		T operator[](difference_type offset) const{
			if(index+offset<0 || (int)vec->dimension()<=index+offset){throw std::logic_error("Iterator out of range");}
			return vec->getElement((unsigned int)index+offset);
		}
		Iterator& operator++() const{
			index++;
			return const_cast<Iterator&>(*this);
		}
		Iterator operator++(int) const{
			Iterator copy = *this;
			index++;
			return copy;
		}
		Iterator& operator--() const{
			index--;
			return const_cast<Iterator&>(*this);
		}
		Iterator operator--(int) const{
			Iterator copy = *this;
			index--;
			return copy;
		}
		Iterator operator+(difference_type off) const {
			Iterator it = *this;
			it.index += off;
			return it;
		}
		Iterator operator-(difference_type off) const {
			Iterator it = *this;
			it.index -= off;
			return it;
		}
		difference_type operator-(const Iterator& rhs) const {
			return index - rhs.index;
		}
		bool operator==(const Iterator &it) const{
			return (vec==it.vec) && (index==it.index);
		}
		bool operator!=(const Iterator &it) const{
			return !(*this==it);
		}
		bool operator<=(const Iterator &it) const{
			return index<=it.index;
		}
		bool operator<(const Iterator &it) const{
			return index<it.index;
		}
		bool operator>=(const Iterator &it) const{
			return index>=it.index;
		}
		bool operator>(const Iterator &it) const{
			return index>it.index;
		}
	private:
		friend class MathVector<T>;
		const MathVector<T> *vec;
		mutable int index;
		mutable ReadWriteRef rwRef;
	};

	/**
	 * Returns an STL-compliant (writable) iterator pointing to the first value.
	 */
	Iterator begin(){
		return Iterator(this,0);
	}
	/**
	 * Returns an STL-compliant (writable) iterator pointing one position after the last element.
	 */
	Iterator end(){
		return Iterator(this,(int)data->dim);
	}

	const Iterator begin() const{
		return Iterator(this,0);
	}
	const Iterator end() const{
		return Iterator(this,(int)data->dim);
	}

// =============================================================================
//    Vector-Vector operations
// =============================================================================

private:
	class VectorAdd{
	public:
		static void evaluate(const MathVector<T> &v1, const MathVector<T> &v2, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = v1.data->values[i] + v2.data->values[i];
			}
		}
	};
	class VectorDiff{
	public:
		static void evaluate(const MathVector<T> &v1, const MathVector<T> &v2, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = v1.data->values[i] - v2.data->values[i];
			}
		}
	};
	class VectorMultScalar{
	public:
		static void evaluate(const MathVector<T> &v, const T &scalar, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = v.data->values[i] * scalar;
			}
		}
	};
	class VectorCWiseMult{
	public:
		static void evaluate(const MathVector<T> &v1, const MathVector<T> &v2, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = v1.data->values[i] * v2.data->values[i];
			}
		}
	};
	class VectorCWiseDiv{
	public:
		static void evaluate(const MathVector<T> &v1, const MathVector<T> &v2, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = v1.data->values[i] / v2.data->values[i];
			}
		}
	};
	class VectorCWiseMax{
	public:
		static void evaluate(const MathVector<T> &v1, const MathVector<T> &v2, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = 
					(v1.data->values[i] < v2.data->values[i]) ? 
						v2.data->values[i] : v1.data->values[i];
			}
		}
	};
	class VectorCWiseMin{
	public:
		static void evaluate(const MathVector<T> &v1, const MathVector<T> &v2, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = 
					(v1.data->values[i] < v2.data->values[i]) ? 
						v1.data->values[i] : v2.data->values[i];
			}
		}
	};


	/*template <class Op>
	class VectorBinaryResult {
	public:
		VectorBinaryResult(const MathVector<T> &v1P, const MathVector<T> &v2P)
		 : v1(v1P), v2(v2P)
		{}

		operator MathVector<T>() const {
			MathVector<T> r(dim(v1.dimension()));
			Op::evaluate(v1, v2, r);
			return r;
		}

	private:
		const MathVector<T> &v1;
		const MathVector<T> &v2;
	};

	template <typename Op>
	MathVector(const VectorBinaryResult<Op> &result)
	 : data(new VectorData(result.v1.data->dim))
	{
		Op::evaluate(result.v1, result.v2, *this);
	}
	template <typename Op>
	MathVector<T>& operator=(const VectorBinaryResult<Op> &result){
		if(data->dim == result.v1.data->dim){
			// TODO this should be uninitialized?
			data.detach();
		}
		else{
			data = new VectorData(result.v1.data->dim);
		}
		assert(data->dim == result.v1.data->dim);
		
		Op::evaluate(result.v1, result.v2, *this);

		return *this;
	}*/


public:
	/**
	 * Adds two vectors of same dimension and returns the result.
	 */
	/*const VectorBinaryResult<VectorAdd> operator+(MathVector<T> &r){
		checkDimensionMatch(r);
		return VectorBinaryResult<VectorAdd>(*this, r);
	}*/
	MathVector<T> operator+(const MathVector<T> &r) const{
		checkDimensionMatch(r);
		MathVector<T> result(dim(data->dim));
		VectorAdd::evaluate(*this, r, result);
		return result;
	}
	/**
	 * Adds a vectors of same dimension.
	 */
	MathVector<T>& operator+=(const MathVector<T> &r){
		checkDimensionMatch(r);
		data.detach();
		VectorAdd::evaluate(*this, r, *this);
		return *this;
	}

	/**
	 * Subtracts two vectors of same dimension and returns the result.
	 */
	/*const VectorBinaryResult<VectorDiff> operator-(MathVector<T> &r) const{
		checkDimensionMatch(r);
		return VectorBinaryResult<VectorDiff>(*this, r);
	}*/
	MathVector<T> operator-(const MathVector<T> &r) const{
		checkDimensionMatch(r);
		MathVector<T> result(dim(data->dim));
		VectorDiff::evaluate(*this, r, result);
		return result;
	}
	/**
	 * Subtracts a vectors of same dimension.
	 */
	MathVector<T>& operator-=(const MathVector<T> &r){
		checkDimensionMatch(r);
		data.detach();
		VectorDiff::evaluate(*this, r, *this);
		return *this;
	}


	/**
	 * Right-multiplies a vector with a scalar and returns the result.
	 */
	friend MathVector<T> operator*(const MathVector<T> &vec, const T &scalar){
		MathVector<T> result(dim(vec.dimension()));
		VectorMultScalar::evaluate(vec, scalar, result);
		return result;
	}
	/**
	 * Left-multiplies a vector with a scalar and returns the result.
	 */
	friend MathVector<T> operator*(const T &scalar, const MathVector<T> &vec){
		MathVector<T> result(dim(vec.dimension()));
		VectorMultScalar::evaluate(vec, scalar, result);
		return result;
	}
	/**
	 * Left-multiplies a vector with a scalar.
	 */
	MathVector<T>& operator*=(const T &scalar){
		data.detach();
		VectorMultScalar::evaluate(*this, scalar, *this);
		return *this;
	}


	/**
	 * Returns a new vector that contains the concatenated entries.
	 * For instance value(1.0).attach(value(2.0,3.0)) returns a 
	 * 3-dimensional vector containing [1.0, 2.0, 3.0].
	 */
	MathVector<T> attach(const MathVector<T> &vec) const {
		//VectorData *newData = new VectorData(data->dim + vec.data->dim);
		MathVector<T> newVec(dim(data->dim + vec.data->dim));
		for(unsigned int i=0; i<data->dim; i++){
			newVec.data->values[i] = data->values[i];
		}
		for(unsigned int i=0; i<vec.data->dim; i++){
			newVec.data->values[data->dim+i] = vec.data->values[i];
		}
		return newVec;
	}

// =============================================================================
//    Vector unary operations
// =============================================================================
private:
	class VectorMinus{
	public:
		static void evaluate(const MathVector<T> &v, MathVector<T> &result){
			for(unsigned int i=0; i<result.data->dim; i++){
				result.data->values[i] = - v.data->values[i];
			}
		}
	};

public:
	MathVector<T> operator-() const{
		MathVector<T> result(dim(data->dim));
		VectorMinus::evaluate(*this, result);
		return result;
	}
	MathVector<T> operator+() const{
		return *this;
	}
		

// =============================================================================
//    Vector-Vector component-wise operations
// =============================================================================

	/**
	 * This class provides component-wise mathematical operations on vectors
	 * that are not covered by standard operators/methods in the MathVector class.
	 */
	class CWise {
	friend class MathVector;
	private:
		CWise(const MathVector<T> &v) : vec(v) {}
	public:
		CWise() : vec() {} // needed for mapping operators!
	private:
		MathVector<T> vec;
	public:
		/** Returns result of component-wise multiplication.
		 */
		MathVector<T> operator*(const MathVector<T> &o) const {
			vec.checkDimensionMatch(o);
			MathVector<T> result(dim(vec.data->dim));
			VectorCWiseMult::evaluate(vec, o, result);
			return result;
		}
		friend MathVector<T> operator*(const MathVector<T> &lhs, const CWise &rhs) {
			if(lhs.dimension() != rhs.vec.dimension()){
				throw std::invalid_argument("");
			}
			MathVector<T> result(dim(lhs.dimension()));

			//VectorCWiseMult::evaluate(lhs, rhs.vec, result);
			// FIXME the above call is not permitted by clang++ (private member, despite friend decl.)
			for(unsigned int i=0; i<result.dimension(); i++){
				result[i] = lhs[i] * rhs.vec[i];
			}

			return result;
		}
		/** Returns result of component-wise division.
		 */
		MathVector<T> operator/(const MathVector<T> &o) const {
			vec.checkDimensionMatch(o);
			MathVector<T> result(dim(vec.data->dim));
			VectorCWiseDiv::evaluate(vec, o, result);
			return result;
		}
		friend MathVector<T> operator/(const MathVector<T> &lhs, const CWise &rhs) {
			if(lhs.dimension() != rhs.vec.dimension()){
				throw std::invalid_argument("");
			}
			MathVector<T> result(dim(lhs.dimension()));
			
			//VectorCWiseDiv::evaluate(lhs, rhs.vec, result);
			// FIXME the above call is not permitted by clang++ (private member, despite friend decl.)
			for(unsigned int i=0; i<result.dimension(); i++){
				result[i] = lhs[i] / rhs.vec[i];
			}
			return result;
		}
		/** Returns component-wise maximum of this vector and the argument.
		 */
		MathVector<T> max(const MathVector<T> &o) const {
			vec.checkDimensionMatch(o);
			MathVector<T> result(dim(vec.data->dim));
			VectorCWiseMax::evaluate(vec, o, result);
			return result;
		}
		/** Returns component-wise minimum of this vector and the argument.
		 */
		MathVector<T> min(const MathVector<T> &o) const {
			vec.checkDimensionMatch(o);
			MathVector<T> result(dim(vec.data->dim));
			VectorCWiseMin::evaluate(vec, o, result);
			return result;
		}
		operator const MathVector<T>() const {return vec;}
	};

	/**
	 * Get a CWise object to perform component-wise operations on this vector.
	 */
	const CWise cwise() const {
		return CWise(*this);
	}
	
	/** Computes component-wise multiplication in place.
	 */
	MathVector<T>& operator*=(const typename MathVector<T>::CWise &o){
		checkDimensionMatch(o.vec);
		data.detach();
		VectorCWiseMult::evaluate(*this, o.vec, *this);
		return *this;
	}
	/** Computes component-wise division in place.
	 */
	MathVector<T>& operator/=(const typename MathVector<T>::CWise &o){
		checkDimensionMatch(o.vec);
		data.detach();
		VectorCWiseDiv::evaluate(*this, o.vec, *this);
		return *this;
	}

private:
	template <typename OutType>
	class CWiseMapping : public MappingDefinition<MathVector<T>,MathVector<OutType> >{
	public:
		MathVector<OutType> evaluate(MathVector<T> value) const{
			return value.map(m);
		}
		CWiseMapping(Mapping<T,OutType> mP) : m(mP) {}
	private:
		Mapping<T,OutType> m;	
	};

public:

	/**
	 * Generates a Vector-valued mapping that corresponds to calling map() on a Vector-instance,
	 * i.e. it takes a scalar Mapping and then applies it component-wise.
	 */ 
	template <typename OutType>
	static Mapping<MathVector<T>,MathVector<OutType> > cwise(Mapping<T,OutType> m){
		return Mapping<MathVector<T>,MathVector<OutType> >(new CWiseMapping<OutType>(m));
	}

// =============================================================================
//    Vector-Vector operations with transpose logic
// =============================================================================

	
	/**
	 * Get a "transpose" object to perform inner or outer product operations.
	 */
	VectorTranspose<T> transpose(){
		return VectorTranspose<T>(*this);
	}

private:

	//inline void _NEMO_VECTOR_CHECK_INDEX(const unsigned int index) const{
	//	if(data->dim <= index){
	//		throw std::invalid_argument("MathVector: index out of range");
	//	}
	//}
	template <class D>
	inline void checkDimensionMatch(const MathVector<D> &r) const{
		if(data->dim != r.data->dim){
			throw std::invalid_argument("MathVector: dimension mismatch");
		}
	}
	
	IntrusiveCowSupport<VectorData > data;
	
};


/**
 * Represents the "transpose" of a vector. 
 */
template <typename T>
class VectorTranspose {
friend class MathVector<T>;
private:
	class VectorScalarProduct{
	public:
		static void evaluate(const MathVector<T> &v1, const MathVector<T> &v2, T &result){
			result = T();
			for(unsigned int i=0; i<v1.data->dim; i++){
				result += v1.data->values[i] * v2.data->values[i];
			}
		}
	};

	VectorTranspose(MathVector<T> &v) : vec(v) {}

	MathVector<T> vec;
public:
	VectorTranspose(const VectorTranspose &o) : vec(o.vec) {}
	VectorTranspose& operator=(const VectorTranspose &o){this->vec=o.vec; return *this;}

	const T& operator[](const unsigned int index) const{
		return vec[index];
	}
	unsigned int dimension() const {
		return vec.dimension();
	}
	/**
	 * Computes the scalar-product of two vectors v1 and v2 by calling v1.transpose()*v2
	 */
	T operator*(const MathVector<T> &o) const {
		vec.checkDimensionMatch(o);
		T result;
		VectorScalarProduct::evaluate(vec, o, result);
		return result;
	}

	MathVector<T> transpose() const {
		return vec;
	}

	bool operator==(const VectorTranspose<T> &r) const{
		if(vec.data->dim != r.vec.data->dim){
			return false;
		}
		for(unsigned int i=0; i<vec.data->dim; i++){
			if(vec.data->values[i] != r.vec.data->values[i]){
				return false;
			}
		}
		// else
		return true;
	}

	/**
	 * ! operator==
	 */
	bool operator!=(const VectorTranspose<T> &r) const{
		return !operator==(r);
	}
};


template <class T>
std::ostream& operator<<(std::ostream& out, const MathVector<T> &vec){

	out << std::string("["); 
	if(0 < vec.dimension()){
		for(unsigned int i=0; i<vec.dimension()-1; i++){
			out << vec[i] << ", ";
		}
		out << vec[vec.dimension()-1];
	}
	out << std::string("]");
	return out;
}
template <class T>
std::ostream& operator<<(std::ostream& out, const VectorTranspose<T> &vecT){
	out << vecT.transpose();
	return out;
}


typedef MathVector<double> RealVector;


// =============================================================================
//    Inversion rules
// =============================================================================

#define _nemo_vector_inversion_(TYPE) \
template <> inline MathVector<TYPE> leftAddInverse<MathVector<TYPE> >(const MathVector<TYPE> &value){return -value;}  \
template <> inline MathVector<TYPE> rightAddInverse<MathVector<TYPE> >(const MathVector<TYPE> &value){return -value;}  \
template <> inline MathVector<TYPE> leftSubtractInverse<MathVector<TYPE> >(const MathVector<TYPE> &value){return value;}  \
template <> inline MathVector<TYPE> rightSubtractInverse<MathVector<TYPE> >(const MathVector<TYPE> &value){return -value;}

_nemo_vector_inversion_(float)
_nemo_vector_inversion_(double)
_nemo_vector_inversion_(long double)
_nemo_vector_inversion_(short)
_nemo_vector_inversion_(int)
_nemo_vector_inversion_(long int)

/*#define _nemo_vector_cwise_inversion_(TYPE) \
template <> inline MathVector<TYPE>::CWise leftMultInverse<>(const MathVector<TYPE>::CWise &value){ \
	return ((MathVector<TYPE>)value).map(((TYPE)1.0)/arg<TYPE>()).cwise(); \
}  \
template <> inline MathVector<TYPE>::CWise rightMultInverse<>(const MathVector<TYPE>::CWise &value){ \
	return ((MathVector<TYPE>)value).map(((TYPE)1.0)/arg<TYPE>()).cwise(); \
}  \
template <> inline MathVector<TYPE>::CWise leftDivideInverse<>(const MathVector<TYPE>::CWise &value){ \
	return value; \
}  \
template <> inline MathVector<TYPE>::CWise rightDivideInverse<>(const MathVector<TYPE>::CWise &value){ \
	return ((MathVector<TYPE>)value).map(((TYPE)1.0)/arg<TYPE>()).cwise(); \
} 

_nemo_vector_cwise_inversion_(float)
_nemo_vector_cwise_inversion_(double)
_nemo_vector_cwise_inversion_(long double)*/


} // end namespace






#endif

